"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util = require("util");
const path = require("path");
const exec = util.promisify(require("child_process").exec);
const fs = require("fs-extra");
function checkDependency(packageName) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield exec(`npm ls ${packageName}`);
            return true;
        }
        catch (e) {
            console.log(e);
            return false;
        }
    });
}
function checkGlobalDependency(packageName) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield exec(`npm ls -g ${packageName}`);
            return true;
        }
        catch (e) {
            console.log(e);
            return false;
        }
    });
}
function hasVersion(pkg) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield exec(`${pkg} --version`);
            return true;
        }
        catch (e) {
            console.log(e);
            return false;
        }
    });
}
function execBuild() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const hasCRA = yield checkDependency("react-scripts");
            if (hasCRA)
                console.log("Building using create-react-app...");
            else
                console.log("Attempting to run 'build' script from package.json");
            const yarnInstalled = yield hasVersion("yarn");
            if (!yarnInstalled) {
                const npm = hasVersion("npm");
                if (!npm)
                    return new Error("NPM is not installed");
                yield exec(`${hasCRA ? "npx react-scripts build" : "npm build"}`);
            }
            yield exec(`yarn ${hasCRA ? "run react-scripts build" : "build"}`);
            return true;
        }
        catch (e) {
            console.log("Error while building application code:\n", e);
            return false;
        }
    });
}
function confirmBuild() {
    return __awaiter(this, void 0, void 0, function* () {
        const pathToCheck = path.join(process.cwd(), "/build");
        if (fs.existsSync(pathToCheck)) {
            return pathToCheck;
        }
        return false;
    });
}
function copyServer(buildPath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const outputDirectory = path.join(process.cwd(), "/build_ipfs");
            const buildOutputDir = path.join(outputDirectory, "/build");
            yield fs.remove(outputDirectory);
            yield fs.mkdirp(outputDirectory);
            yield fs.move(buildPath, buildOutputDir);
            yield fs.copyFile(path.join(process.argv[1], "../../ipfs_server/ipfs.js"), path.join(outputDirectory, "/ipfs.js"));
            yield fs.copyFile(path.join(process.argv[1], "../../ipfs_server/package.json"), path.join(outputDirectory, "/package.json"));
            yield fs.copyFile(path.join(process.argv[1], "../../ipfs_server/package-lock.json"), path.join(outputDirectory, "/package-lock.json"));
        }
        catch (e) {
            console.log("Error copying IPFS server:\n", e);
        }
    });
}
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield execBuild();
            const buildPath = yield confirmBuild();
            yield copyServer(buildPath);
            console.log("Complete!");
        }
        catch (error) {
            console.log(error);
            process.exit(1);
        }
    });
}
main();
exports.default = main;
//# sourceMappingURL=ipfs.js.map